@@ -100,6 +100,7 @@ typedef hash_map<string, macro *, Strhash> macroDictionary;
 typedef hash_map<string, keyword, Strhash> keywordDictionary;
 #endif
 typedef list<plot *> plotSequence;
+typedef vector<double> doublevec;
 typedef list<double> doublelist;
 typedef list<tick *> ticklist;
 typedef list<grid *> gridlist;
@@ -113,7 +114,8 @@ typedef list<shiftdesc *> shiftlist;
 // number of functions taking 0,1,2 args.  The names of those
 // functions are in grap_lex.l and the implementations in the
 // jumptables in grap.y
-const int NF0=1;
+const int NVF1=1;
+const int NF0=2;
 const int NF1=8;
 const int NF2=3;
 
@@ -186,14 +188,17 @@ extern lexStack lexstack;
 #ifndef RANDOM_DECLARED
 extern "C" {
     long random();
+    void srandom(unsigned long);
 }
 #endif
 #else 
 #ifdef HAVE_RAND
 #define random rand
+#define srandom srand
 #ifndef RAND_DECLARED
 extern "C" {
     long rand();
+    void srand(unsigned);
 }
 #endif
 #endif 
