@@ -8,7 +8,7 @@
 #include <stdio.h>
 #include <iostream>
 #include <math.h>
-#ifdef STDC_HEADERS
+#if defined(STDC_HEADERS) | defined(HAVE_STDLIB_H)
 #include <stdlib.h>
 #endif
 #ifdef HAVE_UNISTD_H
@@ -36,8 +36,12 @@ extern bool print_lex_debug;
 
 extern line* defline;
 extern coord *defcoord;
-extern string version; 
 bool compat_mode=false;			//  Compatibility mode
+#ifdef GRAP_SAFER
+bool do_sprintf = false;		//  Allow sprintf
+#else
+bool do_sprintf = true;			//  Allow sprintf
+#endif
 
 // defined in grap_lex.l
 extern int include_string(string *,struct for_descriptor *f=0,
@@ -51,7 +55,7 @@ extern int yyparse();
 extern int nlines;
 
 extern macroDictionary macros;
-const char *opts = "d:lDvuM:CV";
+const char *opts = "d:lDvuM:CVhS";
 
 // Classes for various for_each calls
 
@@ -137,7 +141,7 @@ linedesc* combine_linedesc(linedesc *desc, linedesc* elem) {
 // Process a draw statement.  Create a new line description if this is
 // a new name, assign the line description to it, and a plot string if
 // necessary.
-void draw_statement(string *ident, linedesc *ld, string *plot) {
+void draw_statement(string *ident, linedesc *ld, DisplayString *plot) {
     line *l;
     lineDictionary::iterator li;
     linedesc defld(invis,0,0);
@@ -146,22 +150,25 @@ void draw_statement(string *ident, linedesc *ld, string *plot) {
     if ( ident ) {
 	li = the_graph->lines.find(*ident);
 	if ( li == the_graph->lines.end() ) {
-	    macroDictionary::iterator md;
+	    /*macroDictionary::iterator md;
 	    macro *m;
-	    string *s;
-	
+	    DisplayString *s;
+
 	    // We need to create a new line with default parameters.
 	    // Initialize the line to be invisible with a bullet
 	    // plotting string.
 	    if ( ( md = macros.find("bullet")) != macros.end()) {
 		m = (*md).second;
-		s = m->invoke();
+		string *ss = m->invoke();
+		s = new DisplayString(*ss);
+		delete ss;
 	    }
-	    else s = new string("\"\\(bu\"");
+	    else s = new  DisplayString("\"\\(bu\""); */
 
-	    l = new line(&defld, s);
+	    if ( *ident == "grap_internal_default") defline = l = new line();
+	    else l = new line(*defline);
 	    the_graph->lines[*ident] = l;
-	    delete s;
+	    // delete s;
 	}
 	else {
 	    l = (*li).second;
@@ -180,7 +187,7 @@ void draw_statement(string *ident, linedesc *ld, string *plot) {
     if ( plot ) {
 	if ( *plot != "" ) {
 	    if ( l->plotstr ) *l->plotstr = *plot;
-	    else l->plotstr = new string(*plot);
+	    else l->plotstr = new DisplayString(*plot);
 	}
 	else {
 	    // If the string is "", don't issue the pic commands to
@@ -256,14 +263,7 @@ double assignment_statement(string *ident, double val) {
     delete ident;
     return *d;
 }
-/*
-// Create an new point and return it.  Also note the new point with
-// its coordinate system so that automatic coordinates work right.
-point *new_point(coord *c, double x, double y) {
-    c->newpt(x, y);
-    return new point(x, y, c);
-}
-*/
+
 // This collects the modifiers for other strings in the list to
 // construct the modifiers for the current string.
 stringlist *combine_strings(stringlist *sl, string *st, strmod &sm)  {
@@ -291,22 +291,23 @@ stringlist *combine_strings(stringlist *sl, string *st, strmod &sm)  {
 
 // Create a new plot, that is a string created from a double, and add
 // it to the current graph.
-void plot_statement(double val, string *fmt, point *pt) {
+void plot_statement(double val, DisplayString *fmt, point *pt) {
     stringlist *seq = new stringlist;
     DisplayString *s;
 
-    if ( fmt ) {
+    if ( fmt && do_sprintf ) {
 	unquote(fmt);
 	s = new DisplayString(val,fmt);
-	delete fmt;
     }
     else s = new DisplayString(val);
+   
+    // Delete format whether or not we used it (delete on 0 is OK)
+    delete fmt;
 
     quote(s);
     seq->push_back(s);
 
     the_graph->new_plot(seq,pt);
-    delete pt;
 }
 
 // Add a point to the current line (or the named line if ident is
@@ -321,16 +322,18 @@ void next_statement(string *ident, point *p, linedesc* ld) {
 	if ( li == the_graph->lines.end() ) {
 	    macroDictionary::iterator md;
 	    macro *m;
-	    string *s;
+	    DisplayString *s;
 	
 	    // We need to create a new line with default perameters.
 	    // Initialize the line to be invisible with a bullet
 	    // plotting string.
 	    if ( ( md = macros.find("bullet")) != macros.end()) {
 		m = (*md).second;
-		s = m->invoke();
+		string *ss = m->invoke();
+		s = new DisplayString(*ss);
+		delete ss;
 	    }
-	    else s = new string("\"\\(bu\"");
+	    else s = new DisplayString("\"\\(bu\"");
 	    l = new line((linedesc *)0, s );
 	    the_graph->lines[*ident] = l;
 	    delete s;
@@ -351,16 +354,16 @@ void next_statement(string *ident, point *p, linedesc* ld) {
 // Create a new tick with the given format (if any) and add it to the
 // current ticklist (which we're creating as we go).  If no such list
 // exists, create it.
-ticklist *ticklist_elem(double d, string *fmt, ticklist *tl) {
+ticklist *ticklist_elem(double d, DisplayString *fmt, ticklist *tl) {
     tick *t = new tick(d,0,top_side,0, (shiftlist *) 0, 0);
-    string *s;
+    DisplayString *s;
 
     if ( fmt ) {
 	unquote(fmt);
-	s = dblString(d, fmt);
+	s = new DisplayString(d, fmt);
 	delete fmt;
     }
-    else s = dblString(d);
+    else s = new DisplayString(d);
     t->prt = s;
 
     if ( !tl ) tl = new ticklist;
@@ -372,21 +375,21 @@ ticklist *ticklist_elem(double d, string *fmt, ticklist *tl) {
 // including the coordinate system to put the ticks in, the beginning
 // and ending ticks, a general increment descriptor (by) and a string
 // to format each tick value.  Return the list.
-ticklist *tick_for(coord *c, double from, double to, bydesc by, string *rfmt) {
+ticklist *tick_for(coord *c, double from, double to, bydesc by, DisplayString *rfmt) {
     tick *t;
-    string *s;
-    string *fmt;
+    DisplayString *s;
+    DisplayString *fmt;
     double idx;
     int dir;
     ticklist *tl;
 
     tl = new ticklist;
-    if ( rfmt ) {
+    if ( rfmt && do_sprintf) {
 	unquote(rfmt);
-	fmt = new string(*rfmt);
+	fmt = new DisplayString(*rfmt);
 	delete rfmt;
     } else
-	fmt = new string("%g");
+	fmt = new DisplayString("%g");
 		
     if ( to - from >= 0 ) dir = 1;
     else dir = -1;
@@ -396,7 +399,7 @@ ticklist *tick_for(coord *c, double from, double to, bydesc by, string *rfmt) {
 	t = new tick(idx, 0, top_side, 0, (shiftlist *) 0, 0);
 	t->c = c;
 
-	s = dblString(idx,fmt);
+	s = new DisplayString(idx,fmt);
 	t->prt = s;
 	tl->push_back(t);
 
@@ -532,7 +535,7 @@ void line_statement(int is_line, linedesc *ld1, point *p1,
     // Basically this should never fail...
     li = the_graph->lines.find("grap.internal");
     if ( li == the_graph->lines.end() ) {
-	des.ld = solid;
+	if ( des.ld == def ) des.ld = solid;
 	l = new line(&des);
 	the_graph->lines["grap.internal"] = l;
     }
@@ -575,9 +578,8 @@ void coord_statement(string *ident, axisdesc& xa, axisdesc& ya, axis log) {
     coord *c;
 
     if (ident) {
-	c = new coord;
+	c = new coord(*ident);
 	the_graph->coords[*ident] = c;
-	delete ident;
     } else 
 	c = defcoord;
 
@@ -766,16 +768,14 @@ void bar_statement(coord *c, sides dir, double offset, double ht, double wid,
 // invisible and have a plotting string defined by the bullet macro,
 // if it is defined or a bullet character if not.
 void init_dict() {
-    linedesc defld(invis,0,0);		// The default line descriptor
-    macroDictionary::iterator md;	// An iterator to search for bullet
-    macro *m;				// The bullet macro
-    string *s;				// The plot string for the default line
+    // The plot string for the default line
+    string s = (macros.find("bullet") != macros.end()) ? 
+	"bullet;" : "\"\\(bu\";\n"; 
 
-    if ( ( md = macros.find("bullet")) != macros.end()) {
-	m = (*md).second;
-	s = m->invoke();
-    }
-    else s = new string("\"\\(bu\"");
+    s.insert(0,"new grap_internal_default invis ");
+    include_string(&s, 0, GINTERNAL);
+    defline = new line();
+    the_graph->lines["grap_internal_default"] = defline;
     
     defcoord = new coord;
     the_graph->coords["grap.internal.default"] = defcoord;
@@ -783,10 +783,7 @@ void init_dict() {
 	the_graph->base->tickdef[i].c = defcoord;
 	the_graph->base->griddef[i].c = defcoord;
     }
-    defline = new line(&defld,s);
-    the_graph->lines["grap.internal.default"] = defline;
     nlines = 0;
-    delete s;
 }
 
 
@@ -818,29 +815,62 @@ int yyerror(char *s) {
     return 0;
 }
 
+void usage() {
+    cerr << "Usage: grap [-d defines] [-h|-l|-D|-V|-v|-u|-C|-S] " << 
+	"[-M path] [files]" << endl;
+    cerr << "\t-h\tprint this list and exit (also --help)" << endl;
+    cerr << "\t-d\tuse given defines file" << endl;
+    cerr << "\t-D\tuse no defines file" << endl;
+    cerr << "\t-l\tuse no defines file" << endl;
+    cerr << "\t-v\tprint version and exit (also --version)" << endl;
+    cerr << "\t-V\tprint parse debugging information" << endl;
+    cerr << "\t-u\tforce graph labels to be unaligned by default" << endl;
+    cerr << "\t-M\tspecify search path for files" << endl;
+    cerr << "\t-C\tcompatibility mode" << endl;
+    cerr << "\t-S\tsafer mode, no internal user sprintf calls" << endl;
+    cerr << "See the man page for more information." << endl;
+    exit(5);
+}
+
 int main(int argc, char** argv) {
     string defines=DEFINES;
     string fname;
     string pathstring;
-    int use_defines = 1;
+    bool use_defines = true;
     int c;
 
     if (getenv("GRAP_DEFINES"))
 	defines = getenv("GRAP_DEFINES");
-    
+    if ( getenv("GRAP_SAFER") )
+	do_sprintf = false;
+
+    // Either of these long options are recognized to make the GNU folks
+    // happier.
+    for (int i = 1; i< argc; i++ ) {
+	string av(argv[i]);
+	
+	if ( av == "--version" ) {
+	    cout << "grap " << VERSION << " compiled under ";
+	    cout << OS_VERSION << endl;
+	    exit(5);
+	}
+	else if ( av == "--help" ) usage();
+    }
+
     while ( ( c = getopt(argc,argv,opts)) != -1)
 	switch (c) {
 	    case 'd':
 		defines = optarg;
-		use_defines = 1;
+		use_defines = true;
 		break;
 	    case 'l':
 	    case 'D':
-		use_defines = 0;
+		use_defines = false;
 		break;
 	    case 'v':
-		cout << version << endl;
-		exit(0);
+		cout << "grap " << VERSION << " compiled under ";
+		cout << OS_VERSION << endl;
+		exit(5);
 	    case 'V':
 		print_lex_debug = true;
 		break;
@@ -853,8 +883,13 @@ int main(int argc, char** argv) {
 	    case 'C':
 		compat_mode = true;
 		break;
+	    case 'S':
+		do_sprintf = false;
+		break;
+	    case 'h':
 	    case '?':
-		exit(20);
+		usage();
+		break;
 	}
     pathstring += (pathstring.length() > 0) ? ":." : ".";
 
