@@ -5,10 +5,12 @@
 #endif
 #ifdef STDC_HEADERS
 #include <stdio.h>
-#include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
 #endif
+#if defined(STDC_HEADERS) | defined(HAVE_STDLIB_H)
+#include <stdlib.h>
+#endif
 #include <iostream>
 #include <sys/param.h>
 #include <set> 
@@ -23,7 +25,7 @@ string gk[] = {
     "copy", "next", "draw", "new", "line", "define", "arrow", "circle",
     "plot", "at", "frame", "graph", "coord", "for", "if", "print", "sprintf",
     "ticks", "tick", "label", "grid", "pic", "sh", "bar", "ljust", "rjust",
-    "above", "below", "aligned", "unaligned", "size"
+    "above", "below", "aligned", "unaligned", "size", "undefine"
 };
 
 // These are the keywords recognized by any keyword that takes a line
@@ -40,7 +42,7 @@ string sk[] = {
 // Keywords recognized by ticks and graph
 string tk[] = {
     "left", "right", "up", "down", "bottom", "bot", "top", "in", "out",
-    "from", "to", "by", "at", "off", "on", "auto"
+    "from", "to", "by", "at", "off", "on", "auto", "sprintf"
 };
 
 
@@ -150,7 +152,7 @@ void init_only_once() {
 
     temp.clear();
     temp.push_back("at");
-    keywords["plot"] = keyword(empty, temp, true, PLOT);
+    keywords["plot"] = keyword(temp, empty, true, PLOT);
 }
 
 // These all signal the start of a command
@@ -166,10 +168,12 @@ void init_line_starters() {
 
     temp = linedesc_keys;
     temp.push_back("at");
+    temp.push_back("sprintf");
     keywords["next"] = keyword(temp, empty, true, NEXT);
 
     temp = linedesc_keys;
     temp.insert(temp.end(), strmod_keys.begin(), strmod_keys.end());
+    temp.push_back("sprintf");
     keywords["draw"] = keywords["new"] = keyword(temp, empty, true, DRAW);
     
     temp = linedesc_keys;
@@ -220,6 +224,7 @@ void init_line_starters() {
     keywords["then"] = keyword(temp, empty, true, THEN);
 
     temp = tick_keys;
+    temp.insert(temp.end(), strmod_keys.begin(), strmod_keys.end());
     keywords["tick"] = keywords["ticks"] = keyword(temp, empty, true, TICKS);
 
     temp = strmod_keys;
@@ -230,10 +235,12 @@ void init_line_starters() {
     temp.push_back("bottom");
     temp.push_back("bot");
     temp.push_back("top");
+    temp.push_back("sprintf");
     keywords["label"] = keyword(temp, empty, true, LABEL);
 
     temp = linedesc_keys;
     temp.insert(temp.end(), tick_keys.begin(), tick_keys.end());
+    temp.insert(temp.end(), strmod_keys.begin(), strmod_keys.end());
     temp.push_back("ticks");
     temp.push_back("tick");
     keywords["grid"] = keyword(temp, empty, true, GRID);
@@ -252,6 +259,7 @@ void init_line_starters() {
 void init_wipers() {
     vector<string> empty;
     keywords["define"] = keyword(empty, empty, true, DEFINE);
+    keywords["undefine"] = keyword(empty, empty, true, UNDEFINE);
     keywords["graph"] = keyword(empty, empty, true, GRAPH);
     keywords["else"] = keyword(empty, empty, true, ELSE);
     keywords["print"] = keyword(empty, empty, true, PRINT);
@@ -259,6 +267,24 @@ void init_wipers() {
     keywords["sh"] = keyword(empty, empty, true, SH);
 }
 
+// This are used in parsing multi-element matches for macros.
+
+bool id_letter[256];
+
+// set up the table to scan for embedded macros
+void init_id() {
+    int i;	// Scratch
+    for (i = 0; i < 256; i++ ) id_letter[i] = false;
+
+    // assumes letters are contiugous.  Is there still any character set
+    // for this which doesn't hold?
+    
+    for ( i = 'a'; i <= 'z' ; i++ ) id_letter[i] = true;
+    for ( i = 'A'; i <= 'Z' ; i++ ) id_letter[i] = true;
+    for ( i = '0'; i <= '9' ; i++ ) id_letter[i] = true;
+    id_letter['_'] = true;
+}
+
 // Initialize the table of keywords.
 void init_keywords() {
     vector<string> empty;
@@ -271,4 +297,7 @@ void init_keywords() {
     // Sprintf is unusual in that it doesn't alter the parse state at
     // all.
     keywords["sprintf"] = keyword(empty, empty, false, SPRINTF);
+
+    // init the table
+    init_id();
 }
